@@ -4,8 +4,8 @@
 ;; Keywords: faces files q
 ;; Package-Requires: ((emacs "24"))
 ;; Created: 8 Jun 2015
-;; Package-Version: 20260110.1639
-;; Package-Revision: ce1c5e827c29
+;; Package-Version: 20260215.425
+;; Package-Revision: d37d99d3e917
 ;; URL: https://github.com/psaris/q-mode
 
 ;; This file is not part of GNU Emacs.
@@ -243,11 +243,26 @@ each level is indented by this amount."
   (interactive)
   (q-activate-buffer (current-buffer)))
 
+(defun q-shell-buffer-p (buffer)
+  "Return non-nil if BUFFER is a live Q shell buffer.
+BUFFER can be a buffer object, buffer name, or cons cell from completion."
+  (let ((buf (get-buffer (if (consp buffer) (car buffer) buffer))))
+    (and buf
+         (buffer-live-p buf)
+         (comint-check-proc buf)
+         (with-current-buffer buf
+           (eq major-mode 'q-shell-mode)))))
+
 (defun q-activate-buffer (buffer)
-  "Set the `q-active-buffer' to the supplied BUFFER."
-  (interactive "bactivate buffer: ")
+  "Set the `q-active-buffer' to the supplied BUFFER.
+Prompt with a list of live Q Shell buffers if called interactively."
+  (interactive
+   (list (read-buffer "activate buffer: "
+                      nil
+                      t
+                      #'q-shell-buffer-p)))
   (when (called-interactively-p 'any) (display-buffer buffer))
-  (setq q-active-buffer buffer))
+  (setq q-active-buffer (get-buffer buffer)))
 
 (defun q-default-args ()
   "Build a list of default args out of the q-init customizable variables."
@@ -301,7 +316,7 @@ command to read the command line arguments from the minibuffer."
          (process-environment (cons "KX_LINE=0" process-environment))
          process)
     (when (called-interactively-p 'any) (pop-to-buffer buffer))
-    (when (or current-prefix-arg (not (comint-check-proc buffer)))
+    (when (or current-prefix-arg (not (q-shell-buffer-p buffer)))
       (with-current-buffer buffer
         (message "Starting q with the following command: \"%s\"" cmd)
         (q-shell-mode)
@@ -329,7 +344,7 @@ to read the command line arguments from the minibuffer."
   (let* ((buffer (get-buffer-create (format "*%s*" (format "qcon-%s" args))))
          process)
     (when (called-interactively-p 'any) (pop-to-buffer buffer))
-    (when (or current-prefix-arg (not (comint-check-proc buffer)))
+    (when (or current-prefix-arg (not (q-shell-buffer-p buffer)))
       (with-current-buffer buffer
         (message "Starting qcon with the following cmd: \"%s\"" (concat q-qcon-program " " args))
         (q-shell-mode)
@@ -344,8 +359,7 @@ to read the command line arguments from the minibuffer."
 (defun q-show-q-buffer ()
   "Switch to the active q process, or start a new one (passing in args)."
   (interactive)
-  (unless (and (buffer-live-p q-active-buffer)
-               (comint-check-proc q-active-buffer))
+  (unless (q-shell-buffer-p q-active-buffer)
     (q))
   (if (called-interactively-p 'any)
       (pop-to-buffer q-active-buffer)
